/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <values.h>

#include <qbttngrp.h>
#include <qmsgbox.h> 

#include <kapp.h>
#include <kfiledialog.h>

#include "dialogmargin.h"
#include "printerdialog.h"


CPrinterDialog::CPrinterDialog( const char *title, QWidget *parent, 
			      char *name, bool modal )
  : CPageDialog( CPageDialog::TABS,  CPageDialog::HELP | CPageDialog::PREVIEW |
		 CPageDialog::PRINT | CPageDialog::CANCEL,
		 CPageDialog::PRINT, title, parent, name, modal )
{

  mFrame[ page_destination ] = addPage( -1,i18n("Destination"),
					i18n("Destination"));
  mFrame[ page_option ] = addPage( -1, i18n("Options"), i18n("Options") );
  mFrame[ page_layout ] = addPage( -1, i18n("Page layout"), i18n("Layout") );

  setupDestinationPage();
  setupOptionPage();
  setupLayoutPage();

  adjustSize();
  showPage( page_destination );

  if( parsePrintcap() == FALSE )
  {
    startTimer( 100 );
  }
}



CPrinterDialog::~CPrinterDialog( void )
{
}


void CPrinterDialog::setupDestinationPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_destination ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );  
  if( topLayout == 0 ) { return; }

  mDestination.printerList = new QListView( page, "printerList" );
  mDestination.printerList->setAllColumnsShowFocus( TRUE );
  mDestination.printerList->addColumn( i18n("Printer"), 125 );
  mDestination.printerList->addColumn( i18n("Host"), 125 );
  mDestination.printerList->addColumn( i18n("Comment"), 125 );
  mDestination.printerList->setFrameStyle( QFrame::WinPanel + QFrame::Sunken );
  mDestination.printerList->setMinimumSize( 380, // 125 * 3 + 5 
   mDestination.printerList->fontMetrics().height() * 5 + 1 );


  text = i18n("&Available printers"); 
  mDestination.printerLabel = new QLabel( mDestination.printerList,text,page );
  mDestination.printerLabel->setFixedHeight( 
    mDestination.printerLabel->sizeHint().height() );
  topLayout->addWidget( mDestination.printerLabel );
  topLayout->addWidget( mDestination.printerList, 10 );

  text = i18n("Print to fi&le");
  mDestination.fileCheck = new QCheckBox( text, page );
  mDestination.fileCheck->setFixedHeight( 
    mDestination.fileCheck->sizeHint().height() );
  topLayout->addWidget( mDestination.fileCheck );
  connect( mDestination.fileCheck, SIGNAL(clicked()), SLOT(printFileCheck()) );

 
  QHBoxLayout *hbox = new QHBoxLayout();
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );

  mDestination.fileInput = new QLineEdit( page );
  //Fixed height, see below
  hbox->addWidget( mDestination.fileInput , 0, AlignTop );

  text = i18n("&Browse");
  mDestination.browseButton = new QPushButton( text, page, "browse" );
  mDestination.browseButton->setFixedSize( 
    mDestination.browseButton->sizeHint() );
  mDestination.fileInput->setFixedHeight( 
    mDestination.browseButton->sizeHint().height() );
  hbox->addWidget( mDestination.browseButton, 0, AlignTop );
  connect( mDestination.browseButton, SIGNAL(clicked()),
	   SLOT(browserClicked()) );

  QGridLayout *gbox = new QGridLayout( 1, 2, INNER_SEP );
  if( gbox == 0 ) { return; }
  topLayout->addLayout( gbox );

  QButtonGroup *group = new QButtonGroup( i18n("Print range"), page );
  if( group == 0 ) { return; }
  gbox->addWidget( group, 0, 0 );

  QVBoxLayout *vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );  
  vbox->addSpacing( group->fontMetrics().height() );
  
  QRadioButton *mRadioAll = new QRadioButton( i18n("&Everything"), group );
  mRadioAll->setFixedSize( mRadioAll->sizeHint() );
  group->insert( mRadioAll, 0 );
  vbox->addWidget( mRadioAll, 0, AlignLeft );

  QRadioButton *mRadioSelection = new QRadioButton( i18n("&Selection"), group);
  mRadioSelection->setFixedSize( mRadioSelection->sizeHint() );
  group->insert( mRadioSelection, 1 );
  vbox->addWidget( mRadioSelection, 0, AlignLeft );

  QRadioButton *mRadioRange = new QRadioButton( i18n("&Range"), group );
  mRadioRange->setFixedSize( mRadioRange->sizeHint() );
  group->insert( mRadioRange, 2 );
  vbox->addWidget( mRadioRange, 0, AlignLeft );

  hbox = new QHBoxLayout();
  vbox->addLayout( hbox );
  mDestination.fromInput = new QLineEdit( group );
  mDestination.fromInput->setFixedHeight(
    mDestination.fromInput->sizeHint().height() );
  text = i18n("&From offset"); 
  mDestination.fromLabel = new QLabel( mDestination.fromInput, text, group );
  mDestination.fromLabel->setFixedHeight( 
    mDestination.fromLabel->sizeHint().height() );
  hbox->addWidget( mDestination.fromLabel );
  hbox->addWidget( mDestination.fromInput, 0, AlignLeft );

  hbox = new QHBoxLayout();
  vbox->addLayout( hbox );
  mDestination.toInput = new QLineEdit( group );
  mDestination.toInput->setFixedHeight(
    mDestination.toInput->sizeHint().height() );
  text = i18n("&To offset"); 
  mDestination.toLabel = new QLabel( mDestination.toInput, text, group );
  mDestination.toLabel->setFixedHeight( 
    mDestination.toLabel->sizeHint().height() );
  hbox->addWidget( mDestination.toLabel );
  hbox->addWidget( mDestination.toInput, 0, AlignLeft );
  vbox->addStretch();
  vbox->activate();
  connect( group, SIGNAL(clicked(int)), SLOT(printRangeClicked(int)) );

  group = new QButtonGroup( i18n("Text formatting"), page );
  if( group == 0 ) { return; }
  gbox->addWidget( group, 0, 1 );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );  
  vbox->addSpacing( group->fontMetrics().height() );
  
  text = i18n("Print as postscr&ipt");
  QRadioButton *mPostscript = new QRadioButton( text, group );
  mPostscript->setFixedSize( mPostscript->sizeHint() );
  group->insert( mPostscript, 0 );
  vbox->addWidget( mPostscript, 0, AlignLeft );

  text = i18n("Print as plain &text");
  QRadioButton *mRadioText = new QRadioButton( text, group );
  mRadioText->setFixedSize( mRadioText->sizeHint() );
  group->insert( mRadioText, 1 );
  vbox->addWidget( mRadioText, 0, AlignLeft );
  
  text = i18n("This choice will ignore\ncertain options. Refer to\nhelp for more information.");
  mDestination.helpLabel = new QLabel( text, group );
  mDestination.helpLabel->setFrameStyle( QFrame::Panel | QFrame::Sunken );
  mDestination.helpLabel->setFixedSize( mDestination.helpLabel->sizeHint() );
  mDestination.helpLabel->setAlignment( AlignBottom | AlignLeft );
  vbox->addWidget( mDestination.helpLabel, 0, AlignCenter );

  connect( group, SIGNAL(clicked(int)), SLOT(textFormatClicked(int)) );

  vbox->addStretch();
  vbox->activate();

  mDestination.fileCheck->setChecked( false );
  printFileCheck();
  mRadioAll->setChecked( true );
  printRangeClicked( 0 );
  mPostscript->setChecked( true );
  textFormatClicked( 0 );

  
  topLayout->activate();
}

void CPrinterDialog::setupOptionPage( void )
{
  const char *text;
  QFrame *page = mFrame[ page_option ];

  QVBoxLayout *topLayout = new QVBoxLayout( page, OUTER_SEP, INNER_SEP );  
  if( topLayout == 0 ) { return; }

  QGridLayout *gbox = new QGridLayout( 1, 2, INNER_SEP );
  if( gbox == 0 ) { return; }
  topLayout->addLayout( gbox );

  QButtonGroup *group = new QButtonGroup( i18n("Orientation"), page );
  if( group == 0 ) { return; }
  gbox->addWidget( group, 0, 0 );

  QVBoxLayout *vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );  
  vbox->addSpacing( group->fontMetrics().height() );
  
  QRadioButton *portraitRadio = new QRadioButton( i18n("Por&trait"), group );
  portraitRadio->setFixedSize( portraitRadio->sizeHint() );
  group->insert( portraitRadio, 0 );
  vbox->addWidget( portraitRadio, 0, AlignLeft );

  QRadioButton *landscapeRadio = new QRadioButton( i18n("L&andscape"), group);
  landscapeRadio->setFixedSize( landscapeRadio->sizeHint() );
  group->insert( landscapeRadio, 1 );
  vbox->addWidget( landscapeRadio, 0, AlignLeft );
  vbox->addStretch();
  vbox->activate();


  group = new QButtonGroup( i18n("Colors"), page );
  if( group == 0 ) { return; }
  gbox->addWidget( group, 0, 1 );

  vbox = new QVBoxLayout( group, INNER_SEP, INNER_SEP );  
  vbox->addSpacing( group->fontMetrics().height() );
  
  text = i18n("Print in c&olor");
  QRadioButton *colorRadio = new QRadioButton( text, group);
  colorRadio->setFixedSize( colorRadio->sizeHint() );
  group->insert( colorRadio, 0 );
  vbox->addWidget( colorRadio, 0, AlignLeft );

  text = i18n("Print in &grayscale");
  QRadioButton *grayRadio = new QRadioButton( text, group );
  grayRadio->setFixedSize( grayRadio->sizeHint() );
  group->insert( grayRadio, 1 );
  vbox->addWidget( grayRadio, 0, AlignLeft );

  text = i18n("&Black and white only");
  QRadioButton *bwRadio = new QRadioButton( text, group );
  bwRadio->setFixedSize( bwRadio->sizeHint() );
  group->insert( bwRadio, 1 );
  vbox->addWidget( bwRadio, 0, AlignLeft );
  vbox->addStretch();
  vbox->activate();

  int addSpace = INNER_SEP;
  topLayout->addSpacing( addSpace );

  gbox = new QGridLayout( 3, 3, INNER_SEP+addSpace );
  if( gbox == 0 ) { return; }
  topLayout->addLayout( gbox );
  gbox->setColStretch( 2, 10 );

  QComboBox *paperSelector = new QComboBox( false, page );
  paperSelector->setFixedHeight( paperSelector->sizeHint().height() );
  paperSelector->setMinimumWidth( paperSelector->fontMetrics().width("M")*10 );
  connect( paperSelector, SIGNAL(activated(int)), 
	   SLOT(paperTypeChanged(int)) );

  text = i18n("Paper t&ype");
  QLabel *paperLabel = new QLabel( paperSelector, text, page );
  paperLabel->setFixedHeight( paperSelector->sizeHint().height() );
  paperLabel->setMinimumWidth( paperLabel->sizeHint().width() );

  text = i18n("Size: 8888mm x 8888mm");
  mOption.paperSizeLabel = new QLabel( text, page );
  mOption.paperSizeLabel->setFixedHeight( paperSelector->sizeHint().height() );
  mOption.paperSizeLabel->setMinimumWidth( 
    mOption.paperSizeLabel->sizeHint().width() );
  mOption.paperSizeLabel->hide();
 
  gbox->addWidget( paperLabel, 0, 0 );
  gbox->addWidget( paperSelector, 0, 1 );
  gbox->addWidget( mOption.paperSizeLabel, 0, 2 );

  QSpinBox *pageSpin = new QSpinBox( page );
  pageSpin->setFixedHeight( pageSpin->sizeHint().height() );
  pageSpin->setMinimumWidth( pageSpin->fontMetrics().width("M")*10 );
  pageSpin->setRange( 1, MAXINT );

  text = i18n("&Number of copies");
  QLabel *pageLabel = new QLabel( pageSpin, text, page );
  pageLabel->setFixedHeight( pageSpin->sizeHint().height() );
  pageLabel->setMinimumWidth( pageLabel->sizeHint().width() );
  
  gbox->addWidget( pageLabel, 1, 0 );
  gbox->addWidget( pageSpin, 1, 1 );
 
  group = new QButtonGroup( page );
  if( group == 0 ) { return; }
  group->setFrameStyle( QFrame::NoFrame );
  gbox->addMultiCellWidget( group,  2, 2, 0, 2, AlignLeft );


  QHBoxLayout *hbox = new QHBoxLayout( group, 0, INNER_SEP );
  if( hbox == 0 ) { return; }

  text = i18n("Print &first page first");
  QRadioButton *firstRadio = new QRadioButton( text, group );
  if( firstRadio == 0 ) { return; }
  firstRadio->setFixedSize( firstRadio->sizeHint() );
  group->insert( firstRadio, 0 );
  hbox->addWidget( firstRadio, 0, AlignLeft );

  text = i18n("Print &last page first");
  QRadioButton *lastRadio = new QRadioButton( text, group );
  if( lastRadio == 0 ) { return; }
  lastRadio->setFixedSize( lastRadio->sizeHint() );
  group->insert( lastRadio, 0 );
  hbox->addWidget( lastRadio, 0, AlignLeft );

  hbox->addStretch(10);
  hbox->activate();

  topLayout->addSpacing( addSpace );

  QFrame *hline = new QFrame( page );
  if( hline == 0 ) { return; }
  hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
  hline->setFixedHeight( hline->sizeHint().height() );
  topLayout->addWidget( hline );

  text = i18n("&Scale printed document to fit selected paper type");
  QCheckBox *scaleCheck = new QCheckBox( text, page );
  if( scaleCheck == 0 ) { return; }
  scaleCheck->setFixedSize( scaleCheck->sizeHint() );
  topLayout->addWidget( scaleCheck, 0, AlignLeft );
  
  hbox = new QHBoxLayout( 0 );
  if( hbox == 0 ) { return; }
  topLayout->addLayout( hbox );

  QLineEdit *previewInput = new QLineEdit( page );
  if( previewInput == 0 ) { return; }
  previewInput->setFixedHeight( previewInput->sizeHint().height() );
  hbox->addWidget( previewInput );
 
  text = i18n("Pr&eview application");
  QLabel *previewLabel = new QLabel( previewInput, text, page );
  if( previewLabel == 0 ) { return; }
  previewLabel->setFixedHeight( previewInput->sizeHint().height() );
  previewLabel->setFixedWidth( previewLabel->sizeHint().width() );
  
  hbox->addWidget( previewLabel, 0, AlignLeft );
  hbox->addSpacing( INNER_SEP );
  hbox->addWidget( previewInput, 10 );

  QStrList list;
  paperType( list );
  paperSelector->insertStrList( &list );  

  paperSelector->setCurrentItem( 0 );
  paperTypeChanged( 0 );

  portraitRadio->setChecked( true );
  colorRadio->setChecked( true );
  firstRadio->setChecked( true );

  topLayout->addStretch(10);
  topLayout->activate();
}





void CPrinterDialog::setupLayoutPage( void )
{
}




void CPrinterDialog::printFileCheck( void )
{
  bool state = mDestination.fileCheck->isChecked();
  mDestination.fileInput->setEnabled( state );
  mDestination.browseButton->setEnabled( state );
  state = state == true ? false : true;
  mDestination.printerLabel->setEnabled( state );
  mDestination.printerList->setEnabled( state );
}

void CPrinterDialog::browserClicked( void )
{
  QString url = KFileDialog::getSaveFileName( 0, "*", topLevelWidget() );
  if( url.isEmpty() == true )
  {
    return;
  }

  mDestination.fileInput->setText( url.data() );
}

void CPrinterDialog::printRangeClicked( int id )
{
  bool state = id == 2 ? true : false;
  mDestination.toLabel->setEnabled( state );
  mDestination.fromLabel->setEnabled( state );
  mDestination.toInput->setEnabled( state );
  mDestination.fromInput->setEnabled( state );
}

void CPrinterDialog::textFormatClicked( int id )
{
  mDestination.helpLabel->setEnabled( id == 1 ? true : false );
}


void CPrinterDialog::paperTypeChanged( int id )
{
  QString msg( i18n("Size: ") ); 
  char buf[30];

  const SPageSize &size = paperSize( (QPrinter::PageSize)id );
  sprintf( buf, "%umm x %umm", size.width, size.height );
  msg += buf;

  mOption.paperSizeLabel->setText( msg.data() ); 
  mOption.paperSizeLabel->show();
}




void CPrinterDialog::slotPrint( void )
{
  if( mDestination.fileCheck->isChecked() == false )
  {
    QMessageBox::warning(
      this,
      i18n("Print"),
      "Only to file for now!",
      i18n("OK"),
      "",
      "",
      0,0 );
    return;
  }

  mPrinter.setOutputFileName( mDestination.fileInput->text() );
  

  hide();
  emit printDocument( mPrinter );
}



bool CPrinterDialog::parsePrintcap( void )
{
  bool emptyPrintcap, noPrintcap;
  QString tmp, name;

  char *path[] =
  { 
    "/etc/printers.conf", 
    "/etc/printcap", 
    "/usr/local/etc/printcap",
    0
  };

  emptyPrintcap = noPrintcap = TRUE;

  for( uint i=0; path[i] != 0; i++ )
  {
    QFile file( path[i] );
    if( file.open( IO_ReadOnly ) == FALSE )
    {
      continue;
    }

    noPrintcap = FALSE;

    QTextStream st( (QIODevice *)&file );
    while( st.eof() == FALSE )
    {
      tmp = st.readLine();
      tmp = tmp.stripWhiteSpace();
      if( tmp[0] == '#' || tmp.isEmpty() )
      {
	continue;
      }
      name = tmp.left( tmp.find(':') );
      if( name.isEmpty() )
      {
	continue;
      }
      if( name.contains('|') )
      {
	name = name.left( name.find('|') );
      }
      name = name.stripWhiteSpace();
      if( name.isEmpty() )
      {
	continue;
      }
      else
      {
	emptyPrintcap = FALSE;
      }

      addPrinterName( name );
    }

    file.close();
  }

  return( emptyPrintcap == TRUE ? FALSE : TRUE );
}

void CPrinterDialog::addPrinterName( const QString &printer )
{
  const QListViewItem *entry = mDestination.printerList->firstChild();
  for( ; entry != 0; entry = entry->nextSibling() )
  {
    if( entry->text(0) == printer )
    {
      return;
    }
  }
    
  (void)new QListViewItem( mDestination.printerList, 
			   printer.data(), "host", "comment" );
}

void CPrinterDialog::timerEvent( QTimerEvent *e )
{
  killTimers();
  QString msg( i18n("Unable to parse or read any printcap files" ) );

  QMessageBox::warning(
    this,
    i18n("Print"),
    msg.data(),
    i18n("OK"),
    "",
    "",
    0,0);
}



void CPrinterDialog::paperType( QStrList &list )
{
  list.append(i18n("A4"));
  list.append(i18n("B5"));
  list.append(i18n("Letter"));
  list.append(i18n("Legal"));
  list.append(i18n("Executive"));
  list.append(i18n("A0"));
  list.append(i18n("A1"));
  list.append(i18n("A2"));
  list.append(i18n("A3"));
  list.append(i18n("A5"));
  list.append(i18n("A6"));
  list.append(i18n("A7"));
  list.append(i18n("A8"));
  list.append(i18n("A9"));
  list.append(i18n("B0"));
  list.append(i18n("B1"));
  list.append(i18n("B10"));
  list.append(i18n("B2"));
  list.append(i18n("B3"));
  list.append(i18n("B4"));
  list.append(i18n("B6"));
  list.append(i18n("B7"));
  list.append(i18n("B8"));
  list.append(i18n("B9"));
  list.append(i18n("C5E"));
  list.append(i18n("Comm10E"));
  list.append(i18n("DLE"));
  list.append(i18n("Folio"));
  list.append(i18n("Ledger"));
  list.append(i18n("Tabloid"));
}



const CPrinterDialog::SPageSize &CPrinterDialog::paperSize( 
                            QPrinter::PageSize pageSize )
{
  static SPageSize size[ QPrinter::Tabloid+2 ] = 
  {
    {210,  297},  // A4
    {182,  257},  // B5
    {216,  279},  // Letter
    {216,  356},  // Legal
    {191,  254},  // Executive
    {841,  1189}, // A0  
    {594,  841},  // A1
    {420,  594},  // A2
    {297,  420},  // A3
    {148,  210},  // A5
    {105,  148},  // A6
    {74,   105},  // A7
    {52,   74},   // A8
    {37,   52},   // A9
    {1030, 1456}, // B0 
    {728,  1030}, // B1
    {32,   45},   // B10
    {515,  728},  // B2
    {364,  515},  // B3
    {257,  364},  // B4
    {128,  182},  // B6
    {91,   128},  // B7
    {64,   91},   // B8
    {45,   64},   // B9
    {163,  229},  // C5E
    {105,  241},  // Comm10E
    {110,  220},  // DLE
    {210,  330},  // Folio
    {432,  279},  // Ledger
    {279,  432},  // Tabloid 
    {0,    0},    // Unknown
  };

  return( size[ pageSize > QPrinter::Tabloid ? QPrinter::Tabloid : pageSize ]);
}
