/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *   This file is based on the work by F. Zigterman, fzr@dds.nl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _PRINTER_DIALOG_H_
#define _PRINTER_DIALOG_H_

#include <qframe.h>
#include <qlined.h>
#include <qcombo.h>
#include <qchkbox.h>
#include <kconfig.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <qprinter.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h> 
#include "qlistview.h"

#include "pagedialog.h"



class CPrinterDialog : public CPageDialog
{
  Q_OBJECT

  public:
    enum EPage
    {
      page_destination = 0,
      page_option,
      page_layout,
      page_max,
    };

    struct SPageSize
    {
      unsigned short width;
      unsigned short height;
    };

    CPrinterDialog( const char *title, 
		    QWidget *parent = 0, char *name = 0, bool modal = FALSE );
    ~CPrinterDialog( void );
  public slots:

  signals:
    void printDocument( QPrinter & );

  protected slots:
    virtual void slotPrint( void );
 
  protected:
    void timerEvent( QTimerEvent *e );

  private:
    struct SDestinationWidgets
    {
      QLabel      *printerLabel;
      QListView   *printerList;
      QCheckBox   *fileCheck;
      QLineEdit   *fileInput;
      QPushButton *browseButton;
      QLabel      *toLabel;
      QLabel      *fromLabel;
      QLineEdit   *toInput;
      QLineEdit   *fromInput;
      QLabel      *helpLabel;
    };
    struct SOptionWidgets 
    {
      QLabel      *paperSizeLabel;
    };
    struct SLayoutWidgets
    {
      QWidget *dummy;
    };

  private slots:
    void printFileCheck( void );
    void browserClicked( void );
    void printRangeClicked( int id );
    void textFormatClicked( int id );
    void paperTypeChanged( int id );

  private:
    void setupDestinationPage( void );
    void setupOptionPage( void );
    void setupLayoutPage( void );

    bool parsePrintcap( void );
    void addPrinterName( const QString &printer );
    void paperType( QStrList &list );
    const SPageSize &paperSize( QPrinter::PageSize pageSize );

  private:
    QFrame *mFrame[ page_max ];
    SDestinationWidgets  mDestination;
    SOptionWidgets mOption;
    SLayoutWidgets mLayout;

    QPrinter mPrinter;
};





#endif




