/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *   This file is based on the work by F. Zigterman, fzr@dds.nl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _PROGRESS_H_
#define _PROGRESS_H_

#include "hexerror.h"

typedef int (*ProgressFunc)( void *clientData, float fraction );

class CProgress
{
  public:
    CProgress( void )
    {
      define( 0, 0 );
    }

    CProgress( ProgressFunc progressFunc, void *clientData )
    {
      define( progressFunc, clientData );
    }

    void define( ProgressFunc progressFunc, void *clientData )
    {
      mProgressFunc = progressFunc;
      mClientData = clientData;
    }

    int execute( float progress )
    {
      if( mProgressFunc == 0 )
      {
	return( Err_Success );
      }

      int errCode = mProgressFunc( mClientData, progress );
      return( errCode );
    }

  private:
    ProgressFunc mProgressFunc;
    void *mClientData;
};

#endif

